#!/usr/bin/env python

import sys
import os
from configparser import ConfigParser
from pathlib import Path

import json as js
import click
from prettytable import PrettyTable

from logsight.user import LogsightUser

from logsight_cli.application import application
from logsight_cli.log import log
from logsight_cli.compare import compare
from logsight_cli.incident import incident
from logsight_cli.utils import utils


LOGSIGHT_OPTIONS = ["EMAIL", "PASSWORD", "APP_ID", "DEBUG", "JSON"]
CONFIG = dict.fromkeys(LOGSIGHT_OPTIONS, (None, None))

CONFIG_FILE = os.path.join(str(Path.home()), ".logsight")
cp = ConfigParser()
cp.read(CONFIG_FILE)

CONFIG.update(
    {
        i: (cp["DEFAULT"][i], CONFIG_FILE)
        for i in LOGSIGHT_OPTIONS
        if cp.has_option("DEFAULT", i)
    }
)
CONFIG.update(
    {
        i: (os.environ[f"LOGSIGHT_{i}"], 'Environment')
        for i in LOGSIGHT_OPTIONS
        if f"LOGSIGHT_{i}" in os.environ
    }
)

VERSION = '0.0.18'


@click.group(help="CLI tool to manage logsight.ai artifacts")
@click.version_option(VERSION, prog_name="Logsight CLI")
@click.pass_context
@click.option("--debug/--no-debug")
@click.option("--email", help="User e-mail.")
@click.option("--password", help="User password.")
@click.option("--json", help="Output as JSON.", is_flag=True)
@click.option("--app_id", help="Default app ID.")
def cli(ctx, debug, json, email, password, app_id):

    for k in LOGSIGHT_OPTIONS:
        if locals().get(k.lower()):
            CONFIG[k] = (locals().get(k.lower()), 'Option')

    for k in LOGSIGHT_OPTIONS:
        ctx.obj[k] = CONFIG[k][0]
        if k in ['DEBUG', 'JSON'] and ctx.obj[k] is not None:
            ctx.obj[k] = js.loads(ctx.obj[k].lower())

    ctx.obj["USER"] = LogsightUser(email=ctx.obj["EMAIL"],
                                   password=ctx.obj["PASSWORD"])

    if not ctx.obj["EMAIL"] or not ctx.obj["PASSWORD"]:
        click.echo('Authentication incomplete')
        echo_config()
        sys.exit(1)


@cli.command()
def config():
    """
    Show configuration
    """
    echo_config()
    sys.exit(0)


def echo_config():
    table = PrettyTable(["OPTION", "VALUE", "SOURCE"])
    table.align = "l"
    for i in LOGSIGHT_OPTIONS:
        table.add_row([i, CONFIG[i][0], CONFIG[i][1]])
    click.echo(table)


cli.add_command(application.apps)
cli.add_command(log.log)
cli.add_command(compare.compare)
cli.add_command(incident.incident)
cli.add_command(utils.utils)


def main():
    cli(obj={})


if __name__ == "__main__":
    cli(obj={})



import sys
import json

import click
from prettytable import PrettyTable

from logsight.application import LogsightApplication
from logsight.exceptions import APIException


@click.group("application")
@click.pass_context
def apps(ctx):
    """Manages applications"""


@apps.command()
@click.pass_context
def ls(ctx):
    """
    lists applications registered

    python -m src.logsight_cli application ls
    """
    u = ctx.obj["USER"]

    try:

        app_mng = LogsightApplication(u.user_id, u.token)

        if ctx.obj["JSON"]:
            click.echo(json.dumps(app_mng.lst(), sort_keys=True, indent=4))
        else:
            table = PrettyTable(["APPLICATION ID", "NAME"])
            table.align = "l"
            for a in app_mng.lst()["applications"]:
                table.add_row([a["applicationId"], a["name"]])
            click.echo(table)

    except APIException as e:
        click.echo(f"Unable to retrieve application list ({e})")
        sys.exit(1)

    sys.exit(0)


@apps.command()
@click.pass_context
@click.option("--name", help="name of the application.")
def create(ctx, name):
    """Creates an application

    python -m src.logsight_cli application create --name app_name
    """
    u = ctx.obj["USER"]

    try:

        app_mng = LogsightApplication(u.user_id, u.token)
        r = app_mng.create(name)

        if ctx.obj["JSON"]:
            click.echo(json.dumps(r, sort_keys=True, indent=4))
        else:
            click.echo(f"app_id: {r['applicationId']}")

    except APIException as e:
        click.echo(f"Unable to create application name: {name} ({e})")
        sys.exit(1)

    sys.exit(0)


@apps.command()
@click.pass_context
@click.option("--app_id", help="name of the application.")
def delete(ctx, app_id):
    """Deletes an application

    python -m src.logsight_cli application delete --app_id app_id
    """
    u = ctx.obj["USER"]
    a = app_id or ctx.obj["APP_ID"]

    try:

        app_mng = LogsightApplication(u.user_id, u.token)
        app_mng.delete(a)

    except APIException as e:
        click.echo(f"Unable to delete application name ({e})")
        sys.exit(1)

    sys.exit(0)



from click.testing import CliRunner

from logsight_cli.logsight_cli import cli


AUTH = ['--email', 'jorge.cardoso.pt@gmail.com', '--password', 'pijgu8-jeCjuw-segtes']


def test_config():
    runner = CliRunner()
    result = runner.invoke(cli, AUTH + ['config'], obj={})
    assert result.exit_code == 0
    assert 'EMAIL' in result.output


def test_application():
    runner = CliRunner()
    result = runner.invoke(cli, AUTH + ['application', 'ls'], obj={})
    assert result.exit_code == 0
    assert 'APPLICATION' in result.output


if __name__ == '__main__':
    test_config()



import sys
import json
import time
import click
from tqdm import tqdm
from prettytable import PrettyTable

from logsight.compare import LogsightCompare
from logsight.exceptions import Conflict, BadRequest


N_CALL_RETRIES = 10


@click.group()
@click.pass_context
def compare(ctx):
    """Compares log files"""


@compare.command()
@click.pass_context
@click.option("--app_id", help="id of the application")
@click.option("--tags", required=True, type=click.Tuple([str, str]),
              help="tags to use during comparison")
@click.option("--flush_id", help="flush id")
def log(ctx, app_id, tags, flush_id):
    """
    compare indexed logs

    python -m src.logsight_cli compare log \
    --app_id <applicationId> --tags <tag_v1> <tag_v2> --flush_id <flushId>
    """
    u = ctx.obj["USER"]
    a = app_id or ctx.obj["APP_ID"]

    comp = LogsightCompare(u.user_id, u.token)
    for _ in (
        td := tqdm(
            range(1, N_CALL_RETRIES + 1),
            desc="Call retries",
            colour="white",
            file=sys.stdout,
            disable=not ctx.obj["DEBUG"],
        )
    ):
        td.refresh()
        try:
            r = comp.compare(
                app_id=a,
                baseline_tag=tags[0],
                candidate_tag=tags[1],
                flush_id=flush_id,
                verbose=ctx.obj["DEBUG"],
            )

            if ctx.obj["JSON"]:
                s = json.dumps(r, sort_keys=True, indent=4)
                click.echo(s)
            else:
                table = PrettyTable(["KEY", "VALUE"])
                table.align = "l"
                link = None
                for key, value in r.items():
                    if key != 'link':
                        table.add_row([key, value])
                    else:
                        link = value
                click.echo(table)
                click.echo('Link: ' + link)

            sys.exit(0)
            break
        except Conflict:
            time.sleep(10)
        except BadRequest as e:
            click.echo(e)
            break

    click.echo("Unable to compare log files")
    sys.exit(1)



import sys
import json

import click
from prettytable import PrettyTable

from logsight.logs import LogsightLogs
from logsight.exceptions import APIException
from logsight.compare import LogsightCompare


@click.group()
@click.pass_context
def log(ctx):
    """Manages log files"""


@log.command("upload")
@click.pass_context
@click.argument("file", type=click.Path(exists=True))
@click.option("--tag", help="tag to index the log file.")
@click.option("--app_id", help="application id.")
def upload(ctx, file, tag, app_id):
    """
    Upload a log file to an application

    FILE is the name of the log file

    python -m cli.ls-cli log upload <file> --tag v1 --app_id <applicationId>
    """
    u = ctx.obj["USER"]
    a = app_id or ctx.obj["APP_ID"]

    try:
        logs = LogsightLogs(u.token)
        r = logs.upload(a, file, tag=tag)
        flush_id = logs.flush(r["receiptId"])

        if ctx.obj["JSON"]:
            click.echo(json.dumps(flush_id, sort_keys=True, indent=4))
        else:
            click.echo(f'flush_id: {flush_id["flushId"]}')

    except APIException as e:
        click.echo(f"Unable to upload log file to application ({e})")
        sys.exit(1)

    sys.exit(0)


@log.group('tag')
@click.pass_context
def tag_(ctx):
    """Management of tags"""


@tag_.command()
@click.pass_context
@click.option("--app_id", help="application id.")
def ls(ctx, app_id):
    """
    List the tags of logs

    python -m cli.ls-cli log tag ls --app_id <applicationId>
    """
    u = ctx.obj["USER"]
    a = app_id or ctx.obj["APP_ID"]

    try:

        cmp_mng = LogsightCompare(u.user_id, u.token)

        if ctx.obj["JSON"]:
            click.echo(json.dumps(cmp_mng.tags(a), sort_keys=True, indent=4))
        else:
            table = PrettyTable(["TAG", "VIEW"])
            table.align = "l"
            for i in cmp_mng.tags(a):
                table.add_row([i["tag"], i["tagView"]])
            click.echo(table)

    except APIException as e:
        click.echo(f"Unable to retrieve tags ({e})")
        sys.exit(1)

    sys.exit(0)



import sys
import json
import time
import datetime
import click
from tqdm import tqdm
from prettytable import PrettyTable

from logsight.incidents import LogsightIncident
from logsight.exceptions import Conflict

N_CALL_RETRIES = 10


@click.group()
@click.pass_context
def incident(ctx):
    """Analyzes log files for incidents"""


@incident.command()
@click.pass_context
@click.option("--app_id", help="id of the application")
@click.option(
    "--tag", required=True, type=str, help="tag used to select section of log"
)
@click.option("--flush_id", help="flush id")
def log(ctx, app_id, tag, flush_id):
    """
    Show the incidents that occurred in a log file

    python -m cli.ls-cli incident log --app_id <applicationId> --tag <tag_v1>
    """

    # todo(jcardoso): Tag is not being used. Currently using now() - 1 day
    #   it should be possible to use tags which are resolved to timestamps
    click.echo(
        f"Tag ({tag}) option is not being used."
        f"A default 365d window in being applied"
    )

    u = ctx.obj["USER"]
    a = app_id or ctx.obj["APP_ID"]

    i = LogsightIncident(u.user_id, u.token)
    now = datetime.datetime.utcnow()
    stop_time = now.isoformat()
    start_time = (now - datetime.timedelta(days=365)).isoformat()

    for _ in (
        td := tqdm(
            range(1, N_CALL_RETRIES + 1),
            desc="Call retries",
            colour="white",
            file=sys.stdout,
            disable=not ctx.obj["DEBUG"],
        )
    ):
        td.refresh()
        try:
            r = i.incidents(
                app_id=a,
                start_time=start_time,
                stop_time=stop_time,
                flush_id=flush_id,
                verbose=ctx.obj["DEBUG"],
            )

            if ctx.obj["JSON"]:
                s = json.dumps(r, sort_keys=True, indent=4)
                click.echo(s)
            else:
                table = PrettyTable(["KEY", "VALUE"])
                table.align = "l"
                for key, value in r.items():
                    table.add_row([key, value])
                click.echo(table)

            sys.exit(0)
        except Conflict:
            time.sleep(10)

    click.echo("Unable to retrieve incidents")
    sys.exit(1)



import datetime
from dateutil import parser
from dateutil.tz import tzlocal


def create_log_record(level, message, timestamp=None, metadata=None):
    timestamp = timestamp or datetime.datetime.now(tz=tzlocal()).isoformat()
    return {
        "timestamp": timestamp,
        "level": level,
        "message": message,
        "metadata": metadata or "",
    }


def read_lines(file_name):
    with open(file_name, encoding="utf8") as f:
        lines = f.readlines()
        return lines


def autodetect_datetime(date):
    try:
        return parser.parse(date).replace(tzinfo=tzlocal()).isoformat()
    except parser.ParserError:
        return None


def parse_lines(lines, **kwargs):
    return [parse_line(line, **kwargs) for line in lines]


def parse_line(line, timestamp, level, message):
    sep = " "
    line_lst = line.split()

    d = {
        "timestamp": autodetect_datetime(sep.join(timestamp(line_lst))),
        "level": sep.join(level(line_lst)) or "INFO",
        "message": sep.join(message(line_lst)),
        "metadata": "",
    }

    if not d["timestamp"]:
        return None

    return create_log_record(**d)


def parse_file(file_name, **kwargs):
    lines = read_lines(file_name)
    return [i for i in parse_lines(lines, **kwargs) if i is not None]
    # return lines_to_struct(lines, **kwargs)


if __name__ == "__main__":

    # r = parse_file(file, sep=' ',
    #                timestamp=lambda x: x[1:3],
    #                level=lambda x: x[4:5],
    #                message=lambda x: x[6:])

    FILE = "hadoop_name_node_v2"
    r = parse_file(
        FILE,
        sep=" ",
        timestamp=lambda x: x[0:2],
        level=lambda x: x[2:3],
        message=lambda x: x[3:],
    )
    for i in r:
        print(i)
    print(len(r))



from setuptools import find_packages, setup

VERSION = '0.0.18'


with open("README.rst", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='logsight-cli-py',
    version=VERSION,
    description='Logsight CLI Python',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    author='Jorge Cardoso',
    author_email='jorge.cardoso.pt@gmail.com',
    url="https://github.com/aiops/logsight-cli-py",
    project_urls={
        "Documentation": "https://github.com/aiops/logsight-cli-py/",
        "Source": "https://github.com/aiops/logsight-cli-py",
        "Tracker": "https://github.com/aiops/logsight-cli-py/issues",
    },
    license='unlicense',
    packages=find_packages(exclude=("test",)),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Operating System :: OS Independent",
        "Topic :: Utilities",
    ],
    python_requires=">=3.6",
    install_requires=[
        "logsight-sdk-py",
        "click",
        "prettytable",
        "tqdm",
    ],
    zip_safe=False,
    entry_points={
        "console_scripts": [
            "logsight = logsight_cli.logsight_cli:main",
        ]
    }
)



import click

from logsight_cli.utils.log_parser import parse_line


@click.group()
def utils():
    """Utilities for log transformation"""


@utils.command()
@click.argument("file", type=click.Path(exists=True))
@click.option("--output", required=True, default=" ", type=str,
              help="output file name")
@click.option("--date", required=True, type=click.Tuple([int, int]),
              help="indices of array with the date/time")
@click.option("--level", required=True, type=click.Tuple([int, int]),
              help="indices of array with log level")
@click.option("--message", required=True, type=int,
              help="index of array where message starts")
def transform(file, output, date, level, message):
    """
    transforms the structure of a log file

    FILE, the name of the log file to transform

    e.g.,
    python -m src.logsight_cli utils transform log_file.log \
    --output log_file_out.log --date 0 3 --level 3 3 --message 3
    """
    with open(file, "r", encoding="utf8") as read,\
            open(output, "w", encoding="utf8") as write:
        for line in read:
            data = parse_line(
                line,
                timestamp=lambda x: x[date[0]: date[1]],
                level=lambda x: x[level[0]: level[1]],
                message=lambda x: x[message:],
            )
            if data:
                write.write(
                    " ".join([data[i]
                              for i in ["timestamp", "level", "message"]])
                    + read.newlines
                )
